#include "stdafx.h"
#include "hg_datum.h"

//==============================================================
// Getter des Members nTag
// Das Flag bDirty dient dazu festzulaten, dass die Eingabe
// noch nicht verifiziert wurde. Aufgrund der Reihenfolge-
// problematik wird in der Klasse Datum sptestens vor einer 
// Herausgabe der Daten geprft, ob gecheckt werden muss.
// somit ist es egal, in welcher Reihenfolge die Methoden
// setTag(int pTag), setMonat(int pMonat) und setjahr(int pJahr)
// aufgerufen werden
//==============================================================
int HG::Datum::getTag (void)
{
  if (bDirty) correct();
  return nTag;
}

//==============================================================
// Setter des Members nTag
//==============================================================
void HG::Datum::setTag (int pTag)
{
  bDirty = true;
  nTag = pTag;
}

//==============================================================
// Getter des Members nMonat
//==============================================================
int HG::Datum::getMonat (void)
{
  if (bDirty) correct();
  return nMonat;
}

//==============================================================
// Setter des Members nMonat
//==============================================================
void HG::Datum::setMonat (int pMonat)
{
  bDirty = true;
  nMonat = pMonat;
}

//==============================================================
// Getter des Members nJahr
//==============================================================
int HG::Datum::getJahr (void)
{
  if (bDirty) correct();
  return nJahr;
}

//==============================================================
// Setter des Members nJahr
//==============================================================
void HG::Datum::setJahr (int pJahr)
{
  bDirty = true;
  nJahr = pJahr;
}

//==============================================================
// ganzes Datum auf einmal setzen
//==============================================================
void HG::Datum::set (int pTag, int pMonat, int pJahr)
{
  nTag   = pTag;
  nMonat = pMonat;
  nJahr  = pJahr;
  correct();
}

//==============================================================
// ganzes Datum als long-Wert zurckgeben
//==============================================================
long HG::Datum::getAsLong (void)
{
   if (bDirty) correct();
   return nJahr + nMonat*10000 + nTag * 1000000;
}

//==============================================================
// Datumsregeln prfen
//==============================================================
void HG::Datum::correct (void)
{
   bool bIstSchaltjahr  = false;
   int  nMaxTageImMonat = 31;

   //-----------------------------------------------------------
   // Dirtyflag zurcksetzen
   //-----------------------------------------------------------
   bDirty = false;

   //-----------------------------------------------------------
   // Korrektur: Jahreszahl gltig machen. Das Gregorianische
   // Kalendersystem bersprang 1582 ganze 10 Tage (auf den 
   // 04.10. folgte der 15.10.), um den rechnerischen Kalender
   // (bis dahin Julianisch) wieder mit der Frhlings-
   // Tagundnachtgleiche am 21. Mrz in Einklang zu bringen.
   //-----------------------------------------------------------
   if (nJahr < 1583) nJahr = 1583;
   else if (nJahr > 9999) nJahr = 9999;

   //-----------------------------------------------------------
   // Korrektur: Monatszahl gltig machen 
   //-----------------------------------------------------------
   if (nMonat < 1) nMonat = 1;
   else if (nMonat > 12) nMonat = 12;

   //-----------------------------------------------------------
   // Vorbereitung: feststellen, ob es ein Schaltjahr ist
   //-----------------------------------------------------------
   if (nJahr % 4 == 0)
   {
     if (nJahr % 100 == 0) // Prfung auf Skularjahr
     {
        if (nJahr % 400 == 0) bIstSchaltjahr = true;
     }
     else bIstSchaltjahr = true;
   }

   //-----------------------------------------------------------
   // Vorbereitung: Wieviele Tage der Monat haben kann
   //-----------------------------------------------------------
   switch (nMonat)
   {
     case  2: nMaxTageImMonat = bIstSchaltjahr ? 29 : 28; break;
     case  4:
     case  6:
     case  9:
     case 11: nMaxTageImMonat = 30; break;
     default: nMaxTageImMonat = 31; break;
   }

   //-----------------------------------------------------------
   // Korrektur: Tageszahl gltig machen 
   //-----------------------------------------------------------
   if (nTag < 1) nTag = 1;
   else if (nTag > nMaxTageImMonat) nTag = nMaxTageImMonat;
}