#include "stdafx.h"
#include <time.h>
#include <iostream>
#include <iomanip>
#include "hg_jahrestag.h"

using namespace HG;

//==============================================================
// Standardconstructor, belegt das Datumsobjekt mit dem heutigen
// Datum (gem PC-Uhr)
//==============================================================
Jahrestag::Jahrestag (void) : Datum()
{
  init();
}

//==============================================================
// einfacher Constructor, belegt das Datumsobjekt mit dem heutigen
// Datum (gem PC-Uhr)
//==============================================================
Jahrestag::Jahrestag (std::string pText) : Datum()
{
  init();
  sText = pText;
}

//==============================================================
// Constructor, mit Vorgaben
//==============================================================
Jahrestag::Jahrestag (int pTag, int pMonat, int pJahr) 
: Datum (pTag, pMonat, pJahr)
{
  init();
}

//==============================================================
// Constructor, mit Vorgaben
//==============================================================
Jahrestag::Jahrestag (int pTag, int pMonat, int pJahr, std::string pText) 
: Datum (pTag, pMonat, pJahr)
{
  sText = pText;
}

//==============================================================
// Copy-Constructor (Referenz)
//==============================================================
Jahrestag::Jahrestag (Datum &pDatum)
: Datum (pDatum)
{
  init();
}

//==============================================================
// Copy-Constructor (Pointer)
//==============================================================
Jahrestag::Jahrestag (Datum *ptrDatum) : Datum (ptrDatum)
{
  init();
}

//==============================================================
// Copy-Constructor (Referenz)
//==============================================================
Jahrestag::Jahrestag (Jahrestag &pJahrestag)
: Datum ((Datum)pJahrestag)
{
  sText = pJahrestag.sText;
}

//==============================================================
// Copy-Constructor (Pointer)
//==============================================================
Jahrestag::Jahrestag (Jahrestag *ptrJahrestag)
: Datum ((Datum *)ptrJahrestag)
{
  sText = ptrJahrestag->sText;
}

//==============================================================
// Initialisierung
//==============================================================
void Jahrestag::init (void)
{
  sText = "";
}

//==============================================================
// Rckgabe des Termintextes
//==============================================================
std::string Jahrestag::getText (void)
{
  return sText;
}

//==============================================================
// Zuweisung
//==============================================================
Jahrestag Jahrestag::operator= (Jahrestag &pToCopy)
{
  // Ein einfaches memcpy ber Jahrestag funktioniert nicht,
  // weil string einen Pointer enthlt
   memcpy ((Datum *)this, (Datum *)&pToCopy, sizeof(Datum));
   sText = pToCopy.sText;
   return *this;
}

//==============================================================
// Vergleich
//==============================================================
bool Jahrestag::operator< (Jahrestag &pToCompare)
{
  bool bErgebnis = *((Datum *)this) < (Datum)pToCompare;
  if (bErgebnis) return true;

  bErgebnis = *((Datum *)this) == (Datum)pToCompare;
  if (bErgebnis)
  {
    bErgebnis = sText < pToCompare.sText;
  }
  return bErgebnis;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool Jahrestag::operator<= (Jahrestag &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this > pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool Jahrestag::operator>= (Jahrestag &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this < pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool Jahrestag::operator> (Jahrestag &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !((*this == pToCompare) || (*this < pToCompare));
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool Jahrestag::operator!= (Jahrestag &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  return !(*this == pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool Jahrestag::operator== (Jahrestag &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  bool bErgebnis = memcmp((Datum *)this, (Datum *)&pToCompare, sizeof(Datum)) == 0;
  bErgebnis = bErgebnis && (sText == pToCompare.sText);
  return bErgebnis;
}

//==============================================================
// Operator: Addition mit einem Integer
//==============================================================
Jahrestag Jahrestag::operator+ (int pToAdd)
{
  return Jahrestag (nTag, nMonat, nJahr+pToAdd, sText);
}

//==============================================================
// Operator: Subtraktion eines Integers
//==============================================================
Jahrestag Jahrestag::operator- (int pToSubtract)
{
  return Jahrestag (nTag, nMonat, nJahr-pToSubtract, sText);
}

//==============================================================
// Operator: Prinkrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
Jahrestag Jahrestag::operator++ (void)
{
  (*((Datum*)this))++;
  return *this;
}

//==============================================================
// Operator: Postincrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
Jahrestag Jahrestag::operator++ (int)
{
  Jahrestag oTemp = *this; 
  (*((Datum*)this))++;
  return oTemp;
}

//==============================================================
// Operator: Prdekrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
Jahrestag Jahrestag::operator-- (void)
{
  (*((Datum*)this))--;
  return *this;
}

//==============================================================
// Operator: Postdecrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
Jahrestag Jahrestag::operator-- (int)
{
  Jahrestag oTemp = *this; 
  (*((Datum*)this))--;
  return oTemp;
}

//==============================================================
// Zuweisung
//==============================================================
Jahrestag Jahrestag::operator= (Datum &pToCopy)
{
  (*((Datum*)this)) = pToCopy;
  return *this;
}

//==============================================================
// Zuweisung
//==============================================================
Jahrestag Jahrestag::operator= (std::string &pText)
{
  sText = pText;
  return *this;
}

//==============================================================
// Zuweisung
//==============================================================
Jahrestag Jahrestag::operator= (char *pText)
{
  sText = pText;
  return *this;
}

//**************************************************************
// Nicht-Klassen-Methoden
//**************************************************************
using namespace std;

//==============================================================
// Operator: Addition mit einem Integer (Kommutativ)
//==============================================================
HG::Jahrestag operator+ (int pToAdd, HG::Jahrestag &pTarget)
{
  return pTarget + pToAdd;
}

//==============================================================
// Operator: Subtraktion mit einem Integer - Version 2
//==============================================================
HG::Jahrestag operator- (int pToSubtract, HG::Jahrestag &pJahrestag)
{
  return pJahrestag - pToSubtract;
}

ostream& operator<< (ostream &pStream, HG::Jahrestag &pToShow)
{
  pStream << (HG::Datum)pToShow << " JT: " << pToShow.getText();
  return pStream;
}
